"""

A set of data converters to standardize various covid-19 datasets
into a common format.

"""

from pathlib import Path

import pandas as pd

from .. import helper


_converter_registry = {}


class CaseConverter:
    """Converter factory class."""

    def __init__(self, atlas_folder):
        """Initialze the converter with the path to the atlas"""
        self.atlas_folder = Path(atlas_folder)
        self.converters = []
        self.init_converters()

    def init_converters(self):
        self.converters = [
            cls(self.atlas_folder) for cls in _converter_registry.values()
        ]

    def read_convert(self, path):
        """Reads data from `path` and converts it into the common format."""
        for converter in self.converters:
            if converter.can_convert(path):
                print(f"Using {converter} for {path}")
                return converter.convert(converter.read_data(path))
        raise NotImplementedError(f"{path} could not be read and converted.")


class CaseConverterImpl:
    """Base converter class."""

    conversion_dict = {}
    column_list = []
    common_columns = [
        "date",
        "country",
        "country_label",
        "region_iso",
        "region_label",
        "admin2",
        "admin2_label",
        "tested",
        "positive",
        "deceased",
        "positive_100k",
        "deceased_100k",
    ]

    def __init__(self, atlas_folder):
        """Initialze the converter with the path to the atlas"""
        self.atlas_folder = atlas_folder

    def can_convert(self, path):
        """Returns true if the class can convert the Dataframe."""
        try:
            df = self.read_data(path)
        except (FileNotFoundError, ValueError):
            return False
        return all([col in df.columns for col in self.__class__.column_list])

    def read_data(self, path):
        """Read in the data from a directory path."""
        raise NotImplementedError(
            "Please use one of these subclasses instead:\n"
            "{}".format(", ".join([str(c) for c in _converter_registry]))
        )

    def _set_common_columns(self, df):
        """Use only the common columns; add missing ones when needed."""
        try:
            df["population"] = df.population.astype(int)
        except ValueError:
            pass

        df["positive_100k"] = df["positive"] / df["population"] * 100000
        df["deceased_100k"] = df["deceased"] / df["population"] * 100000

        for column in self.common_columns:
            if column not in df.columns:
                df[column] = ""

        return df[self.common_columns]

    @classmethod
    def _register(cls):
        _converter_registry[cls.__name__] = cls
