"""
Covid-19 converters for data from the covidtracking.com (USA).
"""

from pathlib import Path


import pandas as pd

from . import CaseConverterImpl as CaseConverter
from .. import helper


class CovidtrackingCaseConverter(CaseConverter):
    """
    Converter for data from the United States, collected by
    the Covid tracking project https://covidtracking.com/
    """

    conversion_dict = {
        "totalTestResults": "tested",
        "positive": "positive",
        "death": "deceased",
    }
    column_list = [
        "date",
        "state",
        "positive",
        "negative",
        "pending",
        "hospitalized",
        "death",
        "total",
        "hash",
        "dateChecked",
        "totalTestResults",
        "fips",
        "deathIncrease",
        "hospitalizedIncrease",
        "negativeIncrease",
        "positiveIncrease",
        "totalTestResultsIncrease",
    ]

    def convert(self, df):
        # rename the existing columns
        df_conv = df.rename(columns=CovidtrackingCaseConverter.conversion_dict)

        # convert date
        df_conv["date"] = pd.to_datetime(df_conv["date"], format="%Y%m%d")

        # make states iso-3116 2 compliant
        df_conv["region_iso"] = df_conv.apply(lambda row: f'US-{row["state"]}', axis=1)

        # get population data for US states through right-join type operation
        metadata = pd.read_csv(self.atlas_folder / "wikidata" / "usa-population.csv")
        merged = pd.merge(df_conv, metadata, on="region_iso", how="right")

        # add country information
        merged["country"] = "USA"

        return self._set_common_columns(merged)

    @classmethod
    def read_data(cls, path):
        """Read in the covidtracking state-level data."""
        return pd.read_json(Path(path) / "states-daily.json")


CovidtrackingCaseConverter._register()
