"""
Converters for covid-19 data from Italy.
"""

from pathlib import Path

import pandas as pd

from . import CaseConverterImpl as CaseConverter
from .. import helper


class ItalyCaseConverter(CaseConverter):
    """
    Converter for data from Italian Civil Protection
    and made available at https://github.com/pcm-dpc/COVID-19
    """

    conversion_dict = {
        "data": "date",
        "totale_casi": "positive",
        "tamponi": "tested",
        "deceduti": "deceased",
        "stato": "country",
        "codice_regione": "istatid",
    }

    column_list = ["data", "stato", "codice_regione", "denominazione_regione"]

    def convert(self, df):
        # rename the existing columns
        df_conv = df.rename(columns=ItalyCaseConverter.conversion_dict)

        # get metadata for regions of italy
        metadata = pd.read_csv(self.atlas_folder / 'wikidata' / 'ita-population.csv')
        metadata["istatid"] = metadata.istatid.astype(int)
        df_conv = _correct_trentino(df_conv)
        merged = pd.merge(df_conv, metadata, on="istatid").drop_duplicates()

        return self._set_common_columns(merged)

    def read_data(self, path):
        """Read in the data for Italy."""
        df = pd.read_csv(Path(path) / "dpc-covid19-ita-regioni.csv")
        return df


def _correct_trentino(df):
    """Merge Bolzano and Trento."""
    # The regional data includes Bolzano and Trento separate regions - they are
    # two provinces of the Trentino Alto Adige region, so merge them together

    df_trentino_alto_adige = (
        df.loc[df["istatid"] == 4].groupby("date").sum().reset_index()
    )
    df_trentino_alto_adige["istatid"] = 4
    df_trentino_alto_adige["latitude"] = 46.4337
    df_trentino_alto_adige["longitude"] = 11.1693
    df_trentino_alto_adige["denominazione_regione"] = "Trentino Alto Adige"
    df_trentino_alto_adige["country"] = "ITA"
    df = df[df["istatid"] != 4].append(df_trentino_alto_adige)
    return df


ItalyCaseConverter._register()
