"""
Converters for covid-19 data from JHU-CSSE.
"""

from pathlib import Path

import numpy as np
import pandas as pd

from . import CaseConverterImpl
from .. import helper
import zipfile


class JhuCsseGlobalCaseConverter(CaseConverterImpl):
    """
    Converter for global covid-19 case data from JHU/CSSE
    and made available at https://github.com/CSSEGISandData/COVID-19
    """

    def convert(self, df):
        # combine subregions at the level of country
        df = df.groupby(["date", "Country/Region"]).sum().reset_index()
        df = df.rename({"Country/Region": "region_label"}, axis=1)

        # get population data from worldbank
        zf = zipfile.ZipFile(self.atlas_folder / "worldbank" / "SP.POP.TOTL.zip")
        pop_df = pd.read_csv(
            zf.open("API_SP.POP.TOTL_DS2_en_csv_v2_821007.csv"), skiprows=4
        )

        # Fix the names of countries
        region_jhu_wb_map = {
            "Brunei": "Brunei Darussalam",
            "Czechia": "Czech Republic",
            "Egypt": "Egypt, Arab Rep.",
            "Hong Kong SAR": "Hong Kong SAR, China",
            "Iran": "Iran, Islamic Rep.",
            "Korea, South": "Korea, Rep.",
            "Macao SAR": "Macao SAR, China",
            "Russia": "Russian Federation",
            "Slovakia": "Slovak Republic",
            "Saint Martin": "St. Martin (French part)",
            "US": "United States",
        }
        df = df.replace(region_jhu_wb_map)
        country_code_map = {
            r["Country Name"]: r["Country Code"]
            for i, r in pop_df[["Country Name", "Country Code"]].iterrows()
        }
        df["country"] = df["region_label"].replace(country_code_map)
        pop_ser = pop_df.set_index("Country Code")["2018"]
        merged = df.loc[df["country"].isin(pop_ser.index)].copy()

        # TODO Also consult for worldmap data for countries like Taiwan
        merged["population"] = merged.apply(lambda r: pop_ser.loc[r["country"]], axis=1)
        merged['region_iso'] = merged['country']
        return self._set_common_columns(merged)

    def read_ser(self, path, name):
        """Read in the path and return as a column named name"""
        df = pd.read_csv(path)
        df = df.drop(["Lat", "Long"], axis=1)
        df = df.set_index(["Province/State", "Country/Region"])
        df.columns = pd.to_datetime(df.columns)
        ser = df.stack()
        ser.name = name
        return ser

    def read_data(self, folder):
        """Read in the data."""
        paths = [
            ("time_series_covid19_confirmed_global.csv", "positive"),
            ("time_series_covid19_deaths_global.csv", "deceased"),
        ]
        sers = [self.read_ser(Path(folder) / p, name) for p, name in paths]
        df = pd.concat(sers, axis=1)
        df = df.reset_index().rename({"level_2": "date"}, axis=1)
        return df


# TODO Implement converter for the daily reports

JhuCsseGlobalCaseConverter._register()
