"""
Converters for covid-19 data from Spain.
"""

from pathlib import Path

import pandas as pd

from . import CaseConverterImpl as CaseConverter
from .. import helper

# regional populations extracted from wikidata and wikipedia
# Wikidata is missing information for Aragón
# Query:

# SELECT DISTINCT  ?region_iso ?regionLabel (MAX(?population_cnt) as ?population)
#     {
#         ?country wdt:P298 "ESP" .
#         ?region wdt:P300 ?region_iso .
#         OPTIONAL{?region wdt:P1082 ?population_cnt}
#         ?country wdt:P150 ?region .
#         SERVICE wikibase:label { bd:serviceParam wikibase:language "es" }
#     }
#     GROUP BY ?region_iso ?regionLabel
#     ORDER BY ?regionLabel

region_populations = [
    {"region_iso": "ES-AN", "region_label": "Andalucía", "population": "8409738"},
    {"region_iso": "ES-AR", "region_label": "Aragón", "population": "1308563"},
    {"region_iso": "ES-AS", "region_label": "Asturias", "population": "1028244"},
    {"region_iso": "ES-IB", "region_label": "Baleares", "population": "1150839"},
    {"region_iso": "ES-CN", "region_label": "Canarias", "population": "2127685"},
    {"region_iso": "ES-CB", "region_label": "Cantabria", "population": "580229"},
    {"region_iso": "ES-CM", "region_label": "Castilla-La Mancha", "population": "2106331"},
    {"region_iso": "ES-CL", "region_label": "Castilla y León", "population": "2418694"},
    {"region_iso": "ES-CT", "region_label": "Cataluña", "population": "7619494"},
    {"region_iso": "ES-CE", "region_label": "Ceuta", "population": "84777"},
    {"region_iso": "ES-VC", "region_label": "C. Valenciana", "population": "5003769"},
    {"region_iso": "ES-EX", "region_label": "Extremadura", "population": "1070586"},
    {"region_iso": "ES-GA", "region_label": "Galicia", "population": "2708339"},
    {"region_iso": "ES-MD", "region_label": "Madrid", "population": "6661949"},
    {"region_iso": "ES-ML", "region_label": "Melilla", "population": "86487"},
    {"region_iso": "ES-MC", "region_label": "Murcia", "population": "1478509"},
    {"region_iso": "ES-NA", "region_label": "Navarra", "population": "647554"},
    {"region_iso": "ES-PV", "region_label": "País Vasco", "population": "2167707"},
    {"region_iso": "ES-RI", "region_label": "La Rioja", "population": "315675"},
]


class SpainCaseConverter(CaseConverter):
    """
    Converter for data from https://github.com/datadista/datasets
    """

    conversion_dict = {
        "fecha": "date",
        "total_casos": "positive",
        "total_fallecidos": "deceased",
        "CCAA": "region_label",
    }

    column_list = ["fecha", "CCAA", "cod_ine"]

    def convert(self, df):
        # rename the existing columns
        df_conv = df.rename(columns=SpainCaseConverter.conversion_dict)

        # calculate incidence rates
        merged = df_conv.merge(pd.DataFrame(region_populations))
        merged['country'] = 'ESP'

        return self._set_common_columns(merged)

    def read_data(self, path):
        """Read data for Spain."""
        df_c = pd.read_csv(Path(path) / "ccaa_covid19_casos_long.csv").rename(
            columns={"total": "total_casos"}
        )
        df_f = pd.read_csv(Path(path) / "ccaa_covid19_fallecidos_long.csv").rename(
            columns={"total": "total_fallecidos"}
        )
        return df_c.merge(df_f, how="outer")


SpainCaseConverter._register()
