"""
Covid-19 converters for data from Switzerland.
"""

from pathlib import Path

import pandas as pd

from . import CaseConverterImpl as CaseConverter
from .. import helper


class OpenZHCaseConverter(CaseConverter):
    """
    Converter for data from Switzerland, collected by
    OpenData Zürich and hosted at https://github.com/openZH/covid_19/
    """

    conversion_dict = {
        "ncumul_tested": "tested",
        "ncumul_conf": "positive",
        "ncumul_deceased": "deceased",
        "abbreviation_canton_and_fl": "region_iso",
    }
    column_list = [
        "date",
        "time",
        "abbreviation_canton_and_fl",
        "ncumul_tested",
        "ncumul_conf",
        "ncumul_hosp",
        "ncumul_ICU",
        "ncumul_vent",
        "ncumul_released",
        "ncumul_deceased",
        "source",
    ]

    def convert(self, df):
        # rename the existing columns
        df_conv = df.rename(columns=OpenZHCaseConverter.conversion_dict)

        # make cantons iso-3116 compliant
        df_conv["region_iso"] = df_conv.apply(
            lambda row: f'CH-{row["region_iso"]}', axis=1
        )

        # get metadata for swiss cantons
        metadata = pd.DataFrame(helper.get_region_populations("CHE"))
        merged = pd.merge(df_conv, metadata, on="region_iso").drop_duplicates()
        merged["country"] = "CHE"

        # standardize the canton names
        merged["regionLabel"] = merged.apply(
            lambda row: _standardize_canton_name(row["region_label"]), axis=1
        )

        return self._set_common_columns(merged)

    def read_data(self, path):
        """Read in the swiss cantonal data."""
        df_list = []
        for f in Path(path).glob("COVID19_Fallzahlen_Kanton_*total.csv"):
            df_list.append(pd.read_csv(f))
        df = pd.concat(df_list)
        df["date"] = pd.to_datetime(df["date"], dayfirst=True)
        return df


def _standardize_canton_name(label):
    label = label.lower()
    if label.startswith("canton of"):
        label = label[len("canton of") :].strip()
    return label.capitalize()


OpenZHCaseConverter._register()
