"""Simple functions to help with common charts."""

import altair as alt
import pandas as pd


def make_rule_chart(
    start_case=100,
    max_case=10000,
    max_days=100,
    pos_day=(9, 30000),
    pos_3days=(28, 31000),
    pos_week=(20, 300),
):
    """
    Make dataframe with lines to indicate doubling every day, 3 days, week.

    start_case: case starting point
    max_case: maxium number of cases to plot
    max_days: maximum number of days to show
    pos_day: position of the "doubles every day" label
    pos_3days: position of "doubles every three days" label
    pos_week: position of "doubles every week" label
    """
    days = {"day": range(max_days + 1)}
    logRuleDay_df = pd.DataFrame(days, columns=["day"])
    logRuleDay_df["case"] = start_case * pow(2, logRuleDay_df["day"])
    logRuleDay_df["doubling period"] = "every day"

    logRule3Days_df = pd.DataFrame(days, columns=["day"])
    logRule3Days_df["case"] = start_case * pow(2, (logRule3Days_df["day"]) / 3)
    logRule3Days_df["doubling period"] = "three days"

    logRuleWeek_df = pd.DataFrame(days, columns=["day"])
    logRuleWeek_df["case"] = start_case * pow(2, (logRuleWeek_df["day"]) / 7)
    logRuleWeek_df["doubling period"] = "every week"

    logRules_df = pd.concat([logRuleDay_df, logRule3Days_df, logRuleWeek_df])
    logRules_df = logRules_df.reset_index()

    ruleChart = (
        alt.Chart(logRules_df)
        .mark_line(opacity=0.2, clip=True)
        .encode(
            alt.X("day:Q", scale=alt.Scale(domain=[1, max_days])),
            alt.Y("case", scale=alt.Scale(domain=[start_case, max_case])),
            color=alt.Color("doubling period"),
        )
    )

    # make dataframe for text labels on chart - hand edit these label locations
    textLabels_df = pd.DataFrame(
        [
            [*pos_day, "doubles every day"],
            [*pos_3days, "doubles every 3 days"],
            [*pos_week, "doubles every week"],
        ],
        columns=["labelX", "labelY", "labelText"],
    )

    labelChart = (
        alt.Chart(textLabels_df)
        .mark_text(align="right", baseline="bottom", dx=0, size=15, opacity=0.5)
        .encode(x="labelX", y="labelY", text="labelText")
    )
    return ruleChart + labelChart


def generate_region_chart(
    base,
    column,
    region_column,
    ytitle,
    tooltip_title,
    legend_title=None,
    scale_type="linear",
):
    """
    Produce a regional chart given a column name.

    base: chart base - create with alt.Chart()
    column: name of column to use for y-axis
    region_column: column to use for regions
    ytitle: name of y-axis
    tooltip_title: tooltip title for y-axis values
    legend_title: legend title
    scale_type: "linear" or "log"
    """

    if legend_title is None:
        legend_title = region_column

    color = alt.Color(region_column, legend=alt.Legend(title=legend_title))

    selection = alt.selection_multi(fields=[region_column], bind="legend")
    opacity = alt.condition(selection, alt.value(1), alt.value(0.2))

    tooltip = [
        alt.Tooltip(region_column, title=legend_title),
        alt.Tooltip(column, title=tooltip_title),
        alt.Tooltip("date", title="Date"),
    ]
    chart = (
        base.mark_line()
        .encode(
            alt.X("date", title="Date"),
            alt.Y(column, title=ytitle, scale=alt.Scale(type=scale_type)),
            color=color,
            opacity=opacity,
            tooltip=tooltip,
        )
        .add_selection(selection)
    )

    return chart


def make_region_since_chart(
    base,
    column,
    time_column,
    region_column,
    xtitle,
    ytitle,
    tooltip_title,
    legend_title,
):
    """
    Make chart "days since X case".

    base: chart base - create with alt.Chart()
    column: name of column to use for y-axis
    time_column: column that encodes the time since X case
    region_column: column to use for regions
    xtitle: as it says on the tin
    ytitle: as it says on the tin
    tooltip_title: as it says on the tin
    legend_title: as it says on the tin
    scale_type: "linear" or "log"
    """

    selection = alt.selection_multi(fields=[region_column], bind="legend")
    opacity = alt.condition(selection, alt.value(1), alt.value(0.2))

    lineChart = (
        base.mark_line()
        .encode(
            alt.X(time_column, axis=alt.Axis(title=xtitle)),
            alt.Y(column, axis=alt.Axis(title=ytitle), scale=alt.Scale(type="log"),),
            tooltip=[
                alt.Tooltip(region_column, title=legend_title),
                alt.Tooltip(column, title=tooltip_title),
                alt.Tooltip("date", title="Date"),
            ],
            color=alt.Color(region_column, legend=alt.Legend(title=legend_title)),
            opacity=opacity,
        )
        .add_selection(selection)
    )

    return lineChart
